# -*- coding: utf-8 -*-
"""
Ceci est un script Python du jeu de la reussite de cartes
"familles de couleurs" avec une interface graphique Pygame.
Il n'y a qu'un seul joueur qui doit suivre les insdtructions.
La reussite est terminee lorsque la colonne des cartes 'sept' est complete.
Deux cas peuvent se presenter :
    - Toutes les cartes sont vues de face. C'est une victoire.
    - Au moins une carte est vue de dos. D'est une defaite.

Un bouton "jeu automatique" permet de faire jouer l'ordianteur automatiquement.

"""

# ----- Importation des modules -----
import sys
import pygame
import premier_module as M01
#  M01 contient les fonctions :
#      obtenir_paquet_32_cartes(couleurs, valeurs)
#      melange(jeu)
#      formate(liste_chaines)
import deuxieme_module as M02
#  M02 contient les fonctions :
#  construit_dico_emplacement(couleurs, valeurs)
#  affichage_console(matrice_chaines, matrice_masque)
import troisieme_module as M03
#  M03 contient les fonctions :
#  detection_victoire(matrice_masque)
#  detection_jeu_termine(matrice_chaines)

# Pour desactiver des alertes inapropriees dans Pylint :
# pylint: disable=C0103
# pylint: disable=E1101

#  ----- Constantes utilisees dans ce module -----
#  LARGEUR est la largeur en pixels de la fenetre Pygame.
#  HAUTEUR est la hauteur en pixels de la fenetre Pygame.
#  COTE_CASE est le cote en pixels de haque case du plateau de jeu.
#  VERT_TAPIS, BLANC, JAUNE, JAUNE_FONCE, GRIS_CLAIR, NOIR sont des triplets
#  d'intensite de couleurs RVB.
#  COULEURS, VALEURS sont les listes des 4 couleurs et des 8 valeurs de cartes.
#  COUP_PIOCHE, COUP_ECHANGE valent respectivement 1 et 2.
#  Ce sont les valeurs possibles de la variable coup_actuel.
from constantes import VERT_TAPIS, COULEURS, VALEURS, COUP_PIOCHE,\
    COUP_ECHANGE, LARGEUR, HAUTEUR, BLANC, JAUNE, NOIR, GRIS_CLAIR,\
    JAUNE_FONCE, COTE_CASE

# ----- Declaration des fonctions -----
##############################################################################
#  1. Initialiser une nouvelle partie.
##############################################################################
def nouvelle_partie(paquet):
    '''
    - Elle renvoie la matrice_du_jeu issue du paquet de 32 cartes melangees.
    Cette matrice comporte 40 elements (5 lignes et 8 colonnes).
    Elle represente la grille du jeu affichee dans la fenetre pygame. Les
    cartes sont sur 4 rangees et les 4 cartes du talon sur la 5e rangee.
    - Elle renvoie aussi la matrice matrice_masque (5 lignes et 8 colonnes).
    Cette matrice represente l'etat des cartes sur la grille. Elle contient
    des 0 pour les cartes face visible ou 1 pour dos les cartes dos visible.

    Parameters
    ----------
    paquet : TYPE liste de chaines de caracteres.
        C'est la liste des cartes melangees.
        Par exemple :
        ['h8', 'ck', 'hk', 'c8', 'c7', 'd10', 'hj', 's7', 'sj', 'h9', 'c9',\
         'c1', 's10', 'd1', 's9', 's1', 'dq', 'h10', 'sq', 'h1', 'd7', 'cq',\
         'cj', 's8', 'd9', 'h7', 'dk', 'c10', 'dj', 'hq', 'd8', 'sk']

    Returns
    --------
            matrice_chaines : TYPE liste de listes de chaines de caracteres
            C'est une matrice de 5 lignes de 8 elements. Elle contient des
            cartes 'vide' quand il n'y a pas de carte sur la grille de jeu en
            plus des cartes du paquet.
                Par exemple :
                   [['vide', 'ck', 'sk', 'c10', 'c7', 'sq', 's9', 'dj'],
                    ['vide', 'hj', 'c1', 'd9', 'hk', 'cq', 'h10', 'dk'],
                    ['vide', 'dq', 'd7', 'c8', 'd8', 'c9', 's8', 'h7'],
                    ['vide', 'd1', 'cj', 's1', 'sj', 'h8', 'h9', 'h1'],
                    ['vide', 'vide', 's7', 'd10', 'hq', 's10', 'vide', 'vide']]

            matrice_masque : TYPE liste de listes de chaines de caracteres
            C'est la matrice de 5 lignes de 8 elements qui sont :
              0 si l'emplacement ne contient pas le dos d'une carte.
              1 si l'emplacement contient  le dos d'une carte.

            Au debut d'une nouvelle partie, toutes les cartes sont dos visible.
            Donc la matrice_masque contient 32 nombres 1.
            Les nombres 0 qui restent (cartes face visible) correspondent en
            fait a des emplacements vides sur la grille de jeu. Il s'agit
            de la colonne des cartes 'sept' dans la colonne de gauche et des
            places a gauche et a droite du talon sur la ligne du bas.
            Donc le tableau matrice_masque vaut :
                   [[0, 1, 1, 1, 1, 1, 1, 1],
                    [0, 1, 1, 1, 1, 1, 1, 1],
                    [0, 1, 1, 1, 1, 1, 1, 1],
                    [0, 1, 1, 1, 1, 1, 1, 1],
                    [0, 0, 1, 1, 1, 1, 0, 0]].

    '''

    if M01.formate(paquet) is not None:
        matrice_chaines = M01.formate(paquet)
    else :
        matrice_chaines = []
        ligne = ['vide']*8
        matrice_chaines = [ligne]*5

    matrice_masque = [[0, 1, 1, 1, 1, 1, 1, 1],\
                      [0, 1, 1, 1, 1, 1, 1, 1],\
                      [0, 1, 1, 1, 1, 1, 1, 1],\
                      [0, 1, 1, 1, 1, 1, 1, 1],\
                      [0, 0, 1, 1, 1, 1, 0, 0]]
    return matrice_chaines, matrice_masque

##############################################################################
#  2. Dessiner le plateau de jeu a partir de la matrice qui represente le jeu.
##############################################################################
def dessiner_plateau(matrice_chaines, matrice_masque):
    """
    Avant l'appel a cette fonction, l'etape 1 de Pygame : definition de l'objet
    Surface "fenetre Pygame" par l'instruction pygame.display.set_mode(taille).

    Cette fonction :
        - Efface le contenu de la precedente fenetre Pygame.
        - Realise les ecritures fixes (textes et dessins fixes)
        - Realise les dessins dans la fenetre (cartes vues de face ou de dos
    ou vide qui refletent le contenu des deux matrices de l'etat du jeu.)

    Parameters
    ----------
    matrice_chaines : TYPE liste de listes de chaines de caracteres.
    C'est la premiere matrice de l'etat du jeu.
    En dehors de la fonction elle est nommee matrice_du_jeu.
        Elle est composee de 4 listes de 8 chaines, chacune commencant par
        'vide' (emplacement du sept) et terminee par 7 chaines de caracteres
        (pour 7 cartes du paquet melange).
        Elle est composee enfin d'une cinquieme ligne de 8 chaines pour le
        talon. Il y a sur cette ligne les 4 dernieres cartes du paquet. la
        cinquieme ligne commence par 2 'vide' et termine par 2 'vide'.

        Par exemple :
            [['vide', 'h8', 'hq', 'h10', 'c9', 'c7', 's8', 's1'],
             ['vide', 'c8', 'd8', 'h7', 'd9', 'd10', 'ck', 'dk'],
             ['vide', 'h9', 'd7', 'dq', 'cj', 's9', 'sj', 'h1'],
             ['vide', 'sq', 'hk', 'c1', 'hj', 'cq', 'd1', 'sk'],
             ['vide', 'vide', 'c10', 's7', 'dj', 's10', 'vide', 'vide']]

    matrice_masque : TYPE liste de listes d'entiers.
    C'est la deuxieme matrice de l'etat du jeu.
    En dehors de la fonction elle est nommee matrice_des_dos.
        Elle est composee de 5 listes de 8 entiers.
        Un entier 0 indique qu'il faut afficher la carte face visible.
        Un entier 1 indique qu'il faut afficher la carte dos visible.

        Par exemple :
                 [[0, 1, 1, 1, 0, 1, 1, 0],\
                  [0, 1, 1, 1, 0, 1, 0, 1],\
                  [0, 1, 1, 1, 0, 0, 1, 1],\
                  [0, 1, 1, 1, 0, 1, 1, 1],\
                  [0, 0, 0, 0, 0, 1, 0, 0]]

    Returns
    -------
    None.
    Rien n'est renvoye. C'estune procedure qui met a jour l'affichage dans la
    fenetre Pygame.

    """

    fenetre.fill(VERT_TAPIS)  # Utilise fill pour vider la fenetre.
    passe_ecritures_fixes()

    #  Etape 2 de Pygame : Creer les objets images de Surface :
    #  Trace les 4 lignes principales et la cinquieme pour la pioche.

    #  Conversion de la matrice_de_jeu (remplie de chaines de caractere)
    #  en une matrice d'objets de Surface d'image Pygame grace a la fonction
    #  transforme_matrice(matrice_chaines).
    #  Cela est necessaire pour obtenir des objets "image_carte" dans les
    #  boucles for imbriquees suivantes.
    matrice = transforme_matrice(matrice_chaines)

    #  Pour tenir compte de la matrice_masque.
    #  Si l'element est un 0 dans matrice_masque alors image_carte vaut
    #  l'objet de Surface d'image Pygame contenu dans
    #  matrice[indice_ligne][indice_colonne] (la carte est face visible).
    #  Si l'element est un 1 dans matrice_masque alors image_carte vaut dos
    #  (la carte est dos visible).
    for indice_ligne in range(5):
        for indice_colonne in range (8):
            #  Si la carte doit etre vue de face, on
            #  affecte a la variable image_carte l'objet de Surface image
            #  Pygame stockee aux indice_ligne et indice_colonne precises.
            if matrice_masque[indice_ligne][indice_colonne] == 0:
                image_carte = matrice[indice_ligne][indice_colonne]

            #  sinon on affecte l'objet de Surface image qui correspond au dos.
            elif matrice_masque[indice_ligne][indice_colonne] == 1:
                image_carte = dos

            #  Etape 3. Transfere au bon endroit sur la
            #  fenetre l'objet de surface d'image image_carte qui prend tour
            #  a tour les valeurs enregistrees
            #  dans la matrices du jeu.
            fenetre.blit(image_carte, (indice_colonne*100+100,\
                                       indice_ligne*100+100))
    #  Etape 4 de Pygame : Affichage de la fenetre
    pygame.display.flip()

##############################################################################
#  3. Pour retourner la carte choisie par le joueur si celle-ci est valide
#     c'est a dire est dans la pioche et n'a pas encore ete retournee.
##############################################################################
def jouer_coup_pioche(matrice_chaines, matrice_masque, ligne, colonne):
    """
    - Cette fonction verifie que le joueur a clique sur la
    carte la plus a gauche dans la pioche.

    - Si cette condition est realisee, cette fonction copie dans la
    matrice_chaine la carte cliquee dans la pioche a l'emplacement de la main.
    Puis elle ecrit 'vide' a l'emplacement qu'avait la carte dans la pioche.

    - Cette fonction renvoie False si la case joue par le joueur n'est pas
    jouable c'est a dire que la joueur n'a pas clique la carte la plus a
    gauche dans la pioche.
    Dans le programme principal, cela maintiendra le coup_actif a COUP_PIOCHE.

    - Si le transfert de la carte de la pioche dans la main a ete effectivement
    realise, alors cette fonction renvoie la valeur True pour la variable
    nouveau_coup_pioche_inscrit.

    - Enfin, la fonction examine si la carte qui vient d'etre transferee dans
    la main est un sept (cela sera exploite au niveau des messages dans
    la fenetre Pygame).

    Parameters
    ----------
    matrice_chaines : TYPE liste de listes de listes de chaines caracteres
        Elle memorise l'etat du jeu (le nom des cartes) a chaque instant.

    matrice_masque : TYPE liste de listes de listes d'entiers.
            Elle memorise l'etat du jeu (cartes face visible ou dos visible)
            a chaque instant.

    ligne : TYPE entier.
        C'est le numero de ligne joue par le joueur.

    colonne : TYPE entier.
        C'est le numero de colonne joue par le joueur.

    Returns
    -------
    nouveau_coup_pioche_inscrit : TYPE booleen.
        Vaut True si le coup est valide, False autrement. La valeur est False
        si le joueur clique sur une case autre que la premiere carte a gauche
        dans la pioche.

    un_sept_dans_la_main : TYPE booleen.
        Vaut True si la carte transferee par la fonction depuis la pioche
        vers la main est un sept. Cette information est utile pour la suite
        de l'execution du programme principal.
    """
    nouveau_coup_pioche_inscrit = False
    un_sept_dans_la_main = False

    #  La case est-elle jouable ?
    #  Teste si c'est la ligne de la pioche qui a ete cliquee
    if ligne == 4:
        #  Si oui, on extrait la liste_pioche depuis la ligne 4 de
        #  la matrice de jeu passee dans le parametre matrice_chaines
        liste_pioche = matrice_chaines[4]

        #  On cherche l'indice colonne de la premiere carte non vide dans
        #  la liste pioche en partant de la gauche.
        indice_colonne = 99  # Ce sera la valeur si toute la pioche est vide.
        #  On parcourt la liste pioche depuis la droite, c'est a dire pour
        #  les valeurs d'incice 5, 4, 3, 2.
        for indice in range(5, 1, -1):
            if liste_pioche[indice] != 'vide':
                indice_colonne = indice

        #  Si la pioche est non vide
        if indice_colonne != 99:
            #  On compare la colonne cliquee par le joueur et l'indice colonne
            #  de la premiere carte de la pioche.
            if colonne == indice_colonne:
                # On retire la carte de la pioche et on la place, face visible
                # dans la main c'est a dire dans matrice_chaines[4][7]
                matrice_chaines[4][7] = matrice_chaines[4][indice_colonne]

                # Et matrice_chaines[4][indice_colonne] prend la valeur 'vide'
                matrice_chaines[4][indice_colonne] = 'vide'

        #  matrice des dos est modifiee de facon a ne plus montrer de dos de
        #  carte au niveau de la premiere carte dans la pioche qui a ete
        #  cliquee.
                matrice_masque[4][indice_colonne] = 0

        #  Les modifications dans les deux matrices de l'etat du jeu
        #  (matrice_chaines et matrice_masque) ont ete effectuees donc la
        #  variable nouveau_coup_pioche_inscrit vaut True.
                nouveau_coup_pioche_inscrit = True

    #  Si la chaine de caracteres dans la main est un 7 c'est la dire
    #  si  matrice_chaines[4][7] vaut 's7' ou h7' ou 'd7' ou 'c7'
    if matrice_chaines[4][7][1] == '7':
        un_sept_dans_la_main = True

    #  Si la case cliquee n'est pas jouable c'est a dire n'est pas la premiere
    #  carte de la pioche alors la variable nouveau_coup_pioche_inscrit
    #  est renvoyee avec la valeur False.
    return nouveau_coup_pioche_inscrit, un_sept_dans_la_main

##############################################################################
#  4. Pour echanger la carte qui est dans la main du joueur si celle-ci est
#     valide c'est a dire si les numeros de ligne et de colonne
#     correspondent a la carte passee en parametre.
##############################################################################
def jouer_coup_echange(matrice_chaines, matrice_masque, ligne, colonne, carte):
    """
    Cette fonction renvoie False si la case joue par le joueur n'est pas
    jouable c'est a dire qu'elle ne correspond pas a la ligne et a la colonne
    cliquees. Par exemple il essaye d'echanger la dame de pique en cliquant
    sur une autre position que celle de la dame de pique sur la grille de jeu.

    Si elle correspond a la ligne et a la colonne cliquees alors les matrices
    matrice_chaines, matrice_masque sont mises a jour et la fonction renvoie
    la variable nouveau_coup_echange_inscrit avec la valeur True.

    Parameters
    ----------
    matrice_chaines : TYPE liste de listes de listes de chaines caracteres.
        Elle contient 5 lignes et 8 colonnes.
        Elle memorise l'etat du jeu (le nom des cartes) a chaque instant.

    matrice_masque : TYPE liste de listes de listes d'entiers.
            Elle memorise l'etat du jeu (cartes face visible ou dos visible)
            a chaque instant.

    ligne : TYPE entier.
        C'est le numero de ligne joue par le joueur.

    colonne : TYPE entier.
        C'est le numero de colonne joue par le joueur.

    carte : TYPE chaine de caracteres.
        C'est la carte presente dans la zone "main du jouer". Par exemple 'sq'.

    Returns
    -------
    nouveau_coup_echange_inscrit : TYPE booleen.
        Vaut True si le coup est valide, False autrement. Elle vaut False si
        le joueur clique sur une case qui ne correspond pas a la valeur de la
        carte qui est dans sa main.

    un_sept_dans_la_main : TYPE booleen.
        Vaut True si une carte 7 est dans la main et False sinon.

    nouveau_sept_place : TYPE booleen.
        Vaut True si l'echange qui a ete fait par la fonction a consiste a
        placer une carte sept a sa place sur la grille de jeu.

    """

    nouveau_coup_echange_inscrit = False
    un_sept_dans_la_main = False
    nouveau_sept_place = False

    #  On va avoir besoin de connaitre le couple (ligne, colonne) sur la
    #  grille de jeu qui correspond a la carte presente dans la main du joueur.
    #  Cette information est dans le dictionnaire emplacement qu'on fabrique
    #  a l'aide de la fonction construit_dico_emplacement(COULEURS, VALEURS).
    dico_emplacement = M02.construit_dico_emplacement(COULEURS, VALEURS)

    #  Si le joueur a clique au bon endroit sur la grille pour echanger
    #  la carte qu'il a en main
    if dico_emplacement[carte] == (ligne, colonne):

        #  On echange matrice_chaines[4][7] (la main)
        #  avec matrice_chaines[ligne][colonne]
        tempo = matrice_chaines[ligne][colonne]
        matrice_chaines[ligne][colonne] = matrice_chaines[4][7]
        matrice_chaines[4][7] = tempo

        #  Il reste a ecrire un 0 au lieu d'un 1 dans la matrice_masque.
        #  Ainsi la fonction dessiner_plateau(matrice_chaines, matrice_masque)
        #  dessinera cette carte vue de face au lieu d'etre vue de dos.
        matrice_masque[ligne][colonne] = 0


        #  Les modifications dans les deux matrices de l'etat du jeu
        #  (matrice_chaines et matrice_masque) ont ete effectuees donc la
        #  variable nouveau_coup_echange_inscrit vaut True.
        nouveau_coup_echange_inscrit = True

        #  Si la chaine de caracteres dans la main est un 7 c'est la dire
        #  si  matrice_chaines[4][7] vaut 's7' ou h7' ou 'd7' ou 'c7'
        if matrice_chaines[4][7][1] == '7':
            un_sept_dans_la_main = True

        #  Si la chaine de caracteres dans la case qui a recu la main
        #  est un 7 c'est la dire si
        #  matrice_chaines[ligne][colonne]] vaut 's7' ou h7' ou 'd7' ou 'c7'
        if matrice_chaines[ligne][colonne][1] == '7':
            nouveau_sept_place = True

    return nouveau_coup_echange_inscrit,\
            un_sept_dans_la_main,\
            nouveau_sept_place

##############################################################################
#  5. Pour obtenir des listes d'objets Surface d'image a la place des listes
#     de chaines de caracteres. La fonction dessiner_plateau utilise ce type
#     de listes pour afficher les images des cartes.
##############################################################################
def liste_de_variables(liste_chaines):
    """
    Fonction qui convertit une liste de chaine de caracteres en une liste
    d'objets Surface d'image Pygame

    Parameters
    ----------
    liste_chaines : TYPE liste de chaines de caracteres.
        Par exemple ['s7', 's8'] pour les deux cartes 7 et 8 de pique.

    Returns
    -------
    liste : TYPE liste d'objets de Surface d'image Pygame
        Par exemple [s7, s8] est une liste de deux variables qui designent
        des objets de Surface d'image crees  a partir des images s7.gif, s8.gif

    """

    liste = []
    for element in liste_chaines:
        liste.append(globals()[element])

    return liste


def transforme_matrice(matrice_chaines):
    """
    Transforme une matrice de 5 lignes de 8 chaines de caracteres
    en la meme matrice mais avec des variables de type objet de Surface image.
    On utilise la fonction liste_de_variables qui realise cette transformation
    sur des listes a une dimension.

    Parameters
    ----------
    matrice_chaines : TYPE liste de listes de chaines de caracteres
        Par exemple

    Returns
    -------
    matrice_variables : TYPE liste de listes d'objets Surface d'image de Pygame
        Les elements de la matrice matrice_variables sont par exemple h8 ou d1.
        Ils sont definis plus loin dans le programme juste avant la boucle
        inifinie de jeu, a partir d'images de cates a jouer .gif.

    """

    matrice_variables = []
    for num_ligne in range(5):
        ligne = liste_de_variables(matrice_chaines[num_ligne])
        matrice_variables.append(ligne)
    return matrice_variables


#  Bonus
def jeu_automatique(coup_actuel, matrice_chaines, matrice_masque):
    """
    Cette fonction contient une grande partie du code present dans la boucle
    infinie de jeu.
    Elle contient le code depuis juste apres
    if coup_actif == COUP_PIOCHE and not detection_jeu_termine(matrice_du_jeu):
    jusqu'a la derniere ligne de la boucle infinie de jeu.

    Les differences sont :
        - L'utilisation d'une boucle
        while detection_jeu_termine(matrice_chaines) is not True:
            qui permet la poursuite automatique de la reussite jusqu'a ce
            qu'elle soit termine (par une victoire ou unedefaite).

        - l'absence des messages de consignes au joueur.

        - l'absence des instructions if event.type == pygame.MOUSEBUTTONDOWN:
            puisque le joueur humain de joue pas. Les valeurs des couples
            (ligne, colonne) qui doivent etre passees en parametres aux
            fonctions jouer_coup_pioche et jouer_coup_echange sont calculees
            automatiquement.

    Remarque : les affichages dans la console et dans la fenetre Pygame sont
    maintenus pendant une partie automatique. Celle-ci est lancee en cliquant
    sur le bouton "Jeu automatique" dans la fenetre.

    Parameters
    ----------
    coup_actuel : TYPE entier
        La valeur de cette variable indique si le coup qui est en train d'etre
        execute est du type
        COUP_PIOCHE = 1  # Le joueur doit piocher une carte
        COUP_ECHANGE = 2 # Le joueur doit echanger la carte en main avec une du
                           plateau

    matrice_chaines : TYPE liste de listes de chaines de caracteres.
    C'est la premiere matrice de l'etat du jeu.
    En dehors de la fonction elle est nommee matrice_du_jeu.
        Elle est composee de 4 listes de 8 chaines, chacune commencant par
        'vide' (emplacement du sept) et terminee par 7 chaines de caracteres
        (pour 7 cartes du paquet melange).
        Elle est composee enfin d'une cinquieme ligne de 8 chaines pour le
        talon. Il y a sur cette ligne les 4 dernieres cartes du paquet. la
        cinquieme ligne commence par 2 'vide' et termine par 2 'vide'.
        Par exemple :
            [['vide', 'h8', 'hq', 'h10', 'c9', 'c7', 's8', 's1'],
             ['vide', 'c8', 'd8', 'h7', 'd9', 'd10', 'ck', 'dk'],
             ['vide', 'h9', 'd7', 'dq', 'cj', 's9', 'sj', 'h1'],
             ['vide', 'sq', 'hk', 'c1', 'hj', 'cq', 'd1', 'sk'],
             ['vide', 'vide', 'c10', 's7', 'dj', 's10', 'vide', 'vide']]

    matrice_masque : TYPE liste de listes d'entiers.
    C'est la deuxieme matrice de l'etat du jeu.
    En dehors de la fonction elle est nommee matrice_des_dos.
        Elle est composee de 5 listes de 8 entiers.
        Un entier 0 indique qu'il faut afficher la carte face visible.
        Un entier 1 indique qu'il faut afficher la carte dos visible.
        Par exemple :
                 [[0, 1, 1, 1, 0, 1, 1, 0],\
                  [0, 1, 1, 1, 0, 1, 0, 1],\
                  [0, 1, 1, 1, 0, 0, 1, 1],\
                  [0, 1, 1, 1, 0, 1, 1, 1],\
                  [0, 0, 0, 0, 0, 1, 0, 0]]
    Returns
    -------
    None.
    Il n'y a pas de return. C'est une procedure.

    Remarque : On peut montrer que le variant de boucle "nombre de 'vide'
    dans la colonne des sept" finit par atteindre zero et que donc
    detection_jeu_termine(matrice_chaines) finit par avoir la valeur True.

    """

    pass

#-------------------- Partie Principale ---------------------------
#  Initialisation de tous les modules Pygame. A mettre au debut.
pygame.init()
#  Initialisation de la police pour afficher du texte
pygame.font.init() # Doit etre appele si on veut utiliser pygame.font

#  1. Creation d'un objet de surface d'affichage (c'est la fenetre de jeu).
fenetre = pygame.display.set_mode((LARGEUR, HAUTEUR))  #  On definit la taille.
fenetre.fill(VERT_TAPIS)    #  On remplit de vert.
pygame.display.set_caption("Reussite 'familles de couleurs'")    #  Le titre.

#  Ecritures fixes (textes et dessins) dans la fenetre
def passe_ecritures_fixes():
    """
    On regroupe toutes ces ecritures dans une procedure, car elle doit etre
    appelee chaque fois qu'on efface la fenetre, par exemple dans la fonction
    dessiner_plateau(matrice_chaines, matrice_masque)

    Returns
    -------
    None.
    C'est une procedure qui passe des ecritures fixes dans la fenetre.

    """
    ma_police = pygame.font.SysFont("comicsansms", 15)
    fenetre.blit(ma_police.render("7", True, BLANC), [130,70])
    fenetre.blit(ma_police.render("8", True, BLANC), [230,70])
    fenetre.blit(ma_police.render("9", True, BLANC), [330,70])
    fenetre.blit(ma_police.render("10", True, BLANC), [425,70])
    fenetre.blit(ma_police.render("Valet", True, BLANC), [510,70])
    fenetre.blit(ma_police.render("Dame", True, BLANC), [605,70])
    fenetre.blit(ma_police.render("Roi", True, BLANC), [720,70])
    fenetre.blit(ma_police.render("As", True, BLANC), [820,70])
    fenetre.blit(ma_police.render("Pique", True, BLANC), [880,130])
    fenetre.blit(ma_police.render("Coeur", True, BLANC), [880,230])
    fenetre.blit(ma_police.render("Carreau", True, BLANC), [880,330])
    fenetre.blit(ma_police.render("Trefle", True, BLANC), [880,430])
    fenetre.blit(ma_police.render("PIOCHE", True, JAUNE), [230,530])
    fenetre.blit(ma_police.render("Main du", True, JAUNE), [880,530])
    fenetre.blit(ma_police.render(" joueur", True, JAUNE), [880,560])

    # Dessin de la zone de la main du joueur
    pygame.draw.rect(fenetre, JAUNE, pygame.Rect(798, 498, 75, 100))
    pygame.draw.rect(fenetre, VERT_TAPIS, pygame.Rect(800, 500, 71, 96))
    # Dessin de la zone de la pioche
    pygame.draw.rect(fenetre, JAUNE, pygame.Rect(298, 498, 375, 100))
    pygame.draw.rect(fenetre, VERT_TAPIS, pygame.Rect(300, 500, 371, 96))

    # Dessin du bouton Nouvelle Partie
    pygame.draw.rect(fenetre, NOIR, pygame.Rect(799, 609, 77, 22))
    pygame.draw.rect(fenetre, GRIS_CLAIR, pygame.Rect(800, 610, 75, 20))
    # Ecriture sur le bouton Nouvelle Partie
    ma_police = pygame.font.SysFont("arial", 12)
    fenetre.blit(ma_police.render("Nouvelle partie", True, NOIR), [804,612])

    # Dessin du bouton Jeu automatique
    pygame.draw.rect(fenetre, NOIR, pygame.Rect(799, 639, 77, 22))
    pygame.draw.rect(fenetre, GRIS_CLAIR, pygame.Rect(800, 640, 75, 20))
    # Ecriture sur le bouton Nouvelle Partie
    fenetre.blit(ma_police.render("Jeu automatique", True, NOIR), [803,642])

    # Dessin des quatre images des couleurs aux quatre coins de la fenetre.
    image_pique = pygame.image.load("images/spades.png")
    image_coeur = pygame.image.load("images/hearts.png")
    image_carreau = pygame.image.load("images/diamonds.png")
    image_trefle = pygame.image.load("images/clubs.png")

    fenetre.blit(image_pique, (8, 9))
    fenetre.blit(image_coeur, (953, 6))
    fenetre.blit(image_carreau, (6, 653))
    fenetre.blit(image_trefle, (950, 650))

    # Dessin des traits tout autour de la fenetre
    pygame.draw.line(fenetre, JAUNE_FONCE, [55, 27], [943, 27], 1)
    pygame.draw.line(fenetre, JAUNE_FONCE, [55, 671], [943, 671], 1)
    pygame.draw.line(fenetre, JAUNE_FONCE, [27, 55], [27, 643], 1)
    pygame.draw.line(fenetre, JAUNE_FONCE, [971, 55], [971, 643], 1)

    # Dessin des quatre arcs de cercles jaunes dans les quatre coins de
    # la fenetre
    coin_sup_gauche_diametre_diametre = (0,0,56,56)
    pygame.draw.arc(fenetre, JAUNE_FONCE, coin_sup_gauche_diametre_diametre,\
                    -1.57, 0, 1)
    coin_sup_droit_diametre_diametre = (943,0,56,56)
    pygame.draw.arc(fenetre, JAUNE_FONCE, coin_sup_droit_diametre_diametre,\
                    -3.14, -1.57, 1)
    coin_inf_gauche_diametre_diametre = (0,643,56,56)
    pygame.draw.arc(fenetre, JAUNE_FONCE, coin_inf_gauche_diametre_diametre,\
                    0, 1.57, 1)
    coin_inf_droit_diametre_diametre = (943,643,56,56)
    pygame.draw.arc(fenetre, JAUNE_FONCE, coin_inf_droit_diametre_diametre,\
                    1.57, 3.14, 1)

passe_ecritures_fixes()

#  2. Creation d'objets de type Surface pour peupler le plateau de jeu.
#  Ces objets sont crees a partir d'imeges stockees dans le dossier 'images'.
#  Chaque image de carte est au format .gif et a pour taille 71 x 96 pixels.
vide = pygame.image.load("images/vide.gif").convert() # image verte
dos = pygame.image.load("images/dos.gif").convert() # image de dos de carte
s7 = pygame.image.load("images/s7.gif").convert()
s8 = pygame.image.load("images/s8.gif").convert()
s9 = pygame.image.load("images/s9.gif").convert()
s10 = pygame.image.load("images/s10.gif").convert()
sj = pygame.image.load("images/sj.gif").convert()
sq = pygame.image.load("images/sq.gif").convert()
sk = pygame.image.load("images/sk.gif").convert()
s1 = pygame.image.load("images/s1.gif").convert()
h7 = pygame.image.load("images/h7.gif").convert()
h8 = pygame.image.load("images/h8.gif").convert()
h9 = pygame.image.load("images/h9.gif").convert()
h10 = pygame.image.load("images/h10.gif").convert()
hj = pygame.image.load("images/hj.gif").convert()
hq = pygame.image.load("images/hq.gif").convert()
hk = pygame.image.load("images/hk.gif").convert()
h1 = pygame.image.load("images/h1.gif").convert()
d7 = pygame.image.load("images/d7.gif").convert()
d8 = pygame.image.load("images/d8.gif").convert()
d9 = pygame.image.load("images/d9.gif").convert()
d10 = pygame.image.load("images/d10.gif").convert()
dj = pygame.image.load("images/dj.gif").convert()
dq = pygame.image.load("images/dq.gif").convert()
dk = pygame.image.load("images/dk.gif").convert()
d1 = pygame.image.load("images/d1.gif").convert()
c7 = pygame.image.load("images/c7.gif").convert()
c8 = pygame.image.load("images/c8.gif").convert()
c9 = pygame.image.load("images/c9.gif").convert()
c10 = pygame.image.load("images/c10.gif").convert()
cj = pygame.image.load("images/cj.gif").convert()
cq = pygame.image.load("images/cq.gif").convert()
ck = pygame.image.load("images/ck.gif").convert()
c1 = pygame.image.load("images/c1.gif").convert()

#  ----------------Boucle infinie de jeu ------------------------
#  Initialisations
JEU_TERMINE = False
# Initialisation de la variable coup_actif avant d'entrer pour la premiere
# fois dans la boucle infinie de jeu. En effet le premier coup sera de
# piocher la carte la plus a gauche dans la pioche.
coup_actif = COUP_PIOCHE

# On a beson d'initialiser la variable 'sept_dans_main' a False pour le premier
# passage  dans 'elif coup_actif == COUP_ECHANGE:' plus bas dans le code.
# Cela revient a dire qu'au debut aucune carte '7' n'est dans la main
sept_dans_main = False
nouveau_jeu = M01.obtenir_paquet_32_cartes(COULEURS, VALEURS)
nouveau_paquet = M01.melange(nouveau_jeu)
matrice_du_jeu, matrice_des_dos = nouvelle_partie(nouveau_paquet)
emplacement = M02.construit_dico_emplacement(COULEURS, VALEURS)

#  Double affichage : Matrice dans la console + plateau de jeu Pygame.
M02.affichage_console(matrice_du_jeu, matrice_des_dos)
dessiner_plateau(matrice_du_jeu, matrice_des_dos)

while not JEU_TERMINE:
    ev = pygame.event.get()
    #  On traite les evenements
    for event in ev:
        #  Au cas ou c'est un evenement de fermeture
        if event.type == pygame.QUIT or event.type == pygame.KEYDOWN and\
            event.key == pygame.K_ESCAPE:
            pygame.quit()
            sys.exit()

        #  Cas ou l'evenement est un clic dans le rectangle du bouton
        #  "Nouvelle partie"
        if event.type == pygame.MOUSEBUTTONDOWN:
            #  Capture de l'abscisse de la souris au moment du clic.
            posx = event.pos[0]
            posy = event.pos[1]
            if 799 <= posx <= 876 and 609 <= posy <= 631:
                nouveau_jeu = M01.obtenir_paquet_32_cartes(COULEURS, VALEURS)
                nouveau_paquet = M01.melange(nouveau_jeu)

                #  On obtient une matrice_du_jeu et une matrice_des_dos nouve-
                # les crees a partir d'un nouveau paquet de 32 cartes melangees
                matrice_du_jeu, matrice_des_dos =\
                                                nouvelle_partie(nouveau_paquet)
                #  En cas de lancement d'unenouvelle partie, le coup actif
                #  au depart doit etre le COUP_PIOCHE.
                coup_actif = COUP_PIOCHE

#  Cas ou le coup actif est le COUP_PIOCHE . Ce coup transfere la carte la plus
#  a gauche de la pioche (qui est une carte vue de dos) dans la main.
        if coup_actif == COUP_PIOCHE and\
            not M03.detection_jeu_termine(matrice_du_jeu):
        #  Effacer la zone des messages.
            pygame.draw.rect(fenetre,VERT_TAPIS,(300,600,400,70))
            myfont = pygame.font.SysFont("comicsansms", 15)
            texte_image =\
            myfont.render("Cliquez sur la carte la plus a gauche de la pioche.",\
                                        True,\
                                        BLANC)
            fenetre.blit(texte_image, [330,610])
            pygame.display.update()
            #  Au cas ou c'est un evenement de clic sur la souris
            if event.type == pygame.MOUSEBUTTONDOWN:
                #  Capture de l'abscisse de la souris au moment du clic.
                posx = event.pos[0]
                posy = event.pos[1]
                #  Conversion de l'ordonne de la souris en numero de ligne.
                ligne_choisie = posy//COTE_CASE
                #  Correction de la ligne choisie pour tenir compte que la
                #  zone de 1a ligne en haut de la fenetre occupe les ordonnees
                #  allant de 0 a 100.
                ligne_choisie = ligne_choisie - 1

                #  Conversion de l'abscisse de la souris en numero de colonne.
                colonne_choisie = posx//COTE_CASE
                #  Correction de la colonne choisie pour tenir compte que la
                #  zone de 1a colonne a gauche de la fenetre occupe les
                #  abscisses allant de 0 a 100.
                colonne_choisie = colonne_choisie - 1

                #  Enregistre le coup_pioche s'il est valide (la 1ere carte
                #  dans la pioche).

                #  Enregistre le coup du joueur (s'il est dans une case valide)
                #  dans matrice_du_jeu et matrice_des_dos
                nouveau_coup_pioche_execute, sept_dans_main =\
                                            jouer_coup_pioche(matrice_du_jeu,\
                                            matrice_des_dos,\
                                            ligne_choisie,\
                                            colonne_choisie)
                # Affiche dans la console et dans la fenetre Pygame
                M02.affichage_console(matrice_du_jeu, matrice_des_dos)
                dessiner_plateau(matrice_du_jeu, matrice_des_dos)

                pygame.time.wait(500)
                #  En cas de case choisie valide,
                #  alors le jeu continue avec un coup echange a faire
                if nouveau_coup_pioche_execute is True :
                    coup_actif = COUP_ECHANGE
                else:
                    # Sinon on reste avec le coup pioche a faire.
                    coup_actif = COUP_PIOCHE

#  Cas ou le coup actif est le COUP_ECHANGE. Ce coup transfere la carte
#  la plus a gauche de la pioche (qui est une carte vue de face) dans la grille
#   du jeu a sa bonne place. La carte qui etait vue de dos a cette bonne place
#   est tranferee  immediatement dans la main ou elle est vue de face.
        if coup_actif == COUP_ECHANGE and\
            not M03.detection_jeu_termine(matrice_du_jeu):
            #  Avant de cliquer sur la case de destination pour la carte qu'il
            #  a en main, affichage d'un message de consigne
            #  au joueur selon que la carte dans la main est un '7' ou non
            if sept_dans_main is True:
                #  Effacer la ligne messages.
                pygame.draw.rect(fenetre,VERT_TAPIS,(300,600,400,70))
                #  Preparer les ecritures
                myfont = pygame.font.SysFont("comicsansms", 15)
                texte_image =\
                    myfont.render("Cliquez sur l'emplacement de la carte 7",\
                True,\
                BLANC)
                #  Transferer l'ecriture sur l'objet Surface 'fenetre'
                fenetre.blit(texte_image, [330,610])
                texte_image =\
                    myfont.render("que vous avez en main pour la ranger.",\
                True,\
                BLANC)
                fenetre.blit(texte_image, [330,630])
                #  mettre a jour la fenetre
                pygame.display.update()

            elif sept_dans_main is False:
                pygame.draw.rect(fenetre,VERT_TAPIS,(300,600,400,70))
                myfont = pygame.font.SysFont("comicsansms", 15)
                texte_image =\
                    myfont.render("Cliquez sur l'emplacement de la carte",\
                True,\
                BLANC)
                fenetre.blit(texte_image, [330,610])
                texte_image =\
                    myfont.render("que vous avez en main pour l'echanger.",\
                True,\
                BLANC)
                fenetre.blit(texte_image, [330,630])
                pygame.display.update()

            #  Au cas ou c'est un evenement de clic sur la souris
            if event.type == pygame.MOUSEBUTTONDOWN:
                #  Capture de l'abscisse de la souris au moment du clic.
                posx = event.pos[0]
                posy = event.pos[1]
                #  Conversion de l'ordonne de la souris en numero de ligne.
                ligne_choisie = posy//COTE_CASE
                #  Correction de la ligne choisie pour tenir compte que la
                #  zone de 1a ligne en haut de la fenetre occupe les ordonnees
                #  allant de 0 a 100.
                ligne_choisie = ligne_choisie - 1

                #  Conversion de l'abscisse de la souris en numero de colonne.
                colonne_choisie = posx//COTE_CASE
                #  Correction de la colonne choisie pour tenir compte que la
                #  zone de 1a colonne a gauche de la fenetre occupe les
                #  abscisses allant de 0 a 100.
                colonne_choisie = colonne_choisie - 1

                #  Le if qui suit garantit que le joueur a bien clique
                #  sur une des cartes dans la zone d'echange.
                if 0 <= ligne_choisie <= 3 and 0 <= colonne_choisie <= 7:

                    #  On lit le nom de la carte qui est dans la main du
                    # joueur
                    carte_dans_la_main = matrice_du_jeu[4][7]
# Enregistrement du coup du joueur dans matrice_du_jeu et matrice_des_dos.
                    nouveau_coup_echange_execute, sept_dans_main, sept_place =\
                                    jouer_coup_echange(matrice_du_jeu,\
                                                        matrice_des_dos,\
                                                        ligne_choisie,\
                                                        colonne_choisie,\
                                                        carte_dans_la_main)

#  Affichages au retour de la fonction jouer_coup_echange()
                    M02.affichage_console(matrice_du_jeu, matrice_des_dos)
                    dessiner_plateau(matrice_du_jeu, matrice_des_dos)

                    pygame.time.wait(500)
                    #  En cas de case choisie valide et que le coup d'echange
                    #  s'est termine par le placement d'un sept, le coup
                    #  actif passe a pioche
                    if nouveau_coup_echange_execute and sept_place is True:
                        coup_actif = COUP_PIOCHE

                    else:
                        # Sinon on reste avec le coup echange a faire.
                        coup_actif = COUP_ECHANGE

#  Cas ou l'evenement est un clic dans le rectangle du bouton "Jeu automatique"
        if event.type == pygame.MOUSEBUTTONDOWN:
            #  Capture de l'abscisse de la souris au moment du clic.
            posx = event.pos[0]
            posy = event.pos[1]
            if 799<= posx <= 876 and 639 <= posy <= 661:
                jeu_automatique(coup_actif, matrice_du_jeu, matrice_des_dos)
                print("JEU AUTOMATIQUE TERMINE")
                #  Remarque : A ce niveau le jeu est obligatoirement termine

        if M03.detection_jeu_termine(matrice_du_jeu) is True :
            if M03.detection_victoire(matrice_des_dos):
                pygame.draw.rect(fenetre,\
                                 VERT_TAPIS,\
                                     (300,600,400,70))
                myfont = pygame.font.SysFont("comicsansms", 15)
                texte_image = myfont.render("Victoire !",\
                True,\
                BLANC)
                fenetre.blit(texte_image, [330,610])
                pygame.display.update()
            else:
                pygame.draw.rect(fenetre,\
                                 VERT_TAPIS,\
                                     (300,600,400,70))
                myfont = pygame.font.SysFont("comicsansms", 15)
                texte_image = myfont.render("Defaite !",\
                True,\
                BLANC)
                fenetre.blit(texte_image, [330,610])
                pygame.display.update()
#  Partie qui gere l'apres sortie de boucle infinie de jeu.
pygame.quit()
sys.exit()
